@@ -38,13 +38,29 @@ class _OpRunner:
         return op(*args_, **kwargs_)
 
 
-class ActionNode(Node, ABC):
+class RunnerNode(Node, ABC):
+    @abstractmethod
+    def run(
+        self,
+        payload: dict
+    ) -> typing.Tuple[typing.List[str], any]:
+        """Abstract method to be called when a node's function should be invoked
+
+        Arguments:
+            path {Path} -- The path which contains the other nodes
+            payload {dict} -- Dictionary contianing data to use for node parameters
+        """
+        pass
+
+
+class ActionNode(RunnerNode, ABC):
     """
     Abstract Class to be used for elements within a Path which run functions
     """
 
     def __init__(self, node_def: dict):
-        self._id = node_def.get('id')
+        RunnerNode.__init__(self, node_def)
+        self._return_key = node_def.get('returnKey', None)
         self.node = self.parse_node(node_def.get('block'))
 
     @staticmethod
@@ -94,16 +110,6 @@ class ActionNode(Node, ABC):
         else:
             results_list.append(ret)
 
-    @abstractmethod
-    def run(self, path: ValidFlowPath, payload: dict):
-        """Abstract method to be called when a node's function should be invoked
-
-        Arguments:
-            path {Path} -- The path which contains the other nodes
-            payload {dict} -- Dictionary contianing data to use for node parameters
-        """
-        pass
-
 
 class BooleanNode(ActionNode):
     """
@@ -139,7 +145,7 @@ class BooleanNode(ActionNode):
         self.false_targets = node_def.get('falseTargets')
         self.true_targets = node_def.get('trueTargets')
 
-    def run(self, path: ValidFlowPath, payload: dict):
+    def run(self, payload: dict):
         """Implementation of move on for Boolean Fork.
         calls path.run_node for either the true or false targets,
         based on the result of the inoked opperation for this Fork's onde
@@ -152,11 +158,11 @@ class BooleanNode(ActionNode):
             typing.List[typing.Type[OutputNode]] -- List of out put nodes that are the result
         """
         op_res = self.node.run_op(payload)
+        if self._return_key is not None:
+            payload[self._return_key] = op_res
         targets = self.true_targets if op_res else self.false_targets
-        results = []
-        for target in targets:
-            ActionNode.run_targets(results, target, payload, path)
-        return results, {self.id: op_res}
+        self.is_evalutated = True
+        return targets, op_res
 
 
 class FunctionNode(ActionNode):
@@ -189,17 +195,14 @@ class FunctionNode(ActionNode):
 
     def __init__(self, node_def: dict):
         ActionNode.__init__(self, node_def)
-        self.output_source_node_ids = node_def.get('targets')
-        self.return_key = node_def.get('return_key', None)
+        self.targets = node_def.get('targets')
 
-    def run(self, path: ValidFlowPath, payload: dict):
+    def run(self, payload: dict):
         op_res = self.node.run_op(payload=payload)
-        if self.return_key is not None:
-            payload[self.return_key] = op_res
-        results = []
-        for output in self.output_source_node_ids:
-            ActionNode.run_targets(results, output, payload, path)
-        return results, {self.id: op_res}
+        if self._return_key is not None:
+            payload[self._return_key] = op_res
+        self.is_evalutated = True
+        return self.targets, op_res
 
 
 class LoopStartNode(ActionNode):
